@@ -17,6 +17,12 @@ find_globals_conservative <- function(expr, envir, ..., trace = FALSE) {
     w <- makeUsageCollector(fun, name = "<anonymous>", enterGlobal = enter)
     if (trace) w <- inject_tracer_to_walker(w)
     collect_usage_function(fun, name = "<anonymous>", w)
+  } else if (is.call(expr) && is.function(expr[[1]])) {
+    ## AD HOC: Fixes https://github.com/HenrikBengtsson/globals/issues/60
+    for (e in list(expr[[1]], expr[-1])) {
+      globals <- find_globals_conservative(expr = e, envir = envir, ..., trace = trace)
+      if (length(globals) > 0) objs <- c(objs, globals)
+    }
   } else {
     ## From codetools::findGlobals():
     fun <- as_function(expr, envir = envir, ...)
@@ -50,6 +56,12 @@ find_globals_liberal <- function(expr, envir, ..., trace = FALSE) {
     w <- makeUsageCollector(fun, name = "<anonymous>", enterGlobal = enter)
     if (trace) w <- inject_tracer_to_walker(w)
     collect_usage_function(fun, name = "<anonymous>", w)
+  } else if (is.call(expr) && is.function(expr[[1]])) {
+    ## AD HOC: Fixes https://github.com/HenrikBengtsson/globals/issues/60
+    for (e in list(expr[[1]], expr[-1])) {
+      globals <- find_globals_liberal(expr = e, envir = envir, ..., trace = trace)
+      if (length(globals) > 0) objs <- c(objs, globals)
+    }
   } else {
     fun <- as_function(expr, envir = envir, ...)
     w <- makeUsageCollector(fun, name = "<anonymous>", enterGlobal = enter)
@@ -93,11 +105,10 @@ find_globals_ordered <- function(expr, envir, ..., trace = FALSE) {
     ## Also walk formulas to identify globals
     if (type == "function") {
       if (v == "~") {
-        stop_if_not(identical(e[[1]], as.symbol("~")))
-        expr <- e[-1]
-        for (kk in seq_along(expr)) {
-          globals <- find_globals_ordered(expr = expr[[kk]], envir = w$env)
-          if (length(globals) > 0) {
+        stop_if_not(length(e) >= 2L, identical(e[[1]], as.symbol("~")))
+        for (kk in 2:length(e)) {
+          globals <- call_find_globals_with_dotdotdot(find_globals_ordered, expr = e[[kk]], envir = w$env, dotdotdot = "return")
+            if (length(globals) > 0) {
             class <<- c(class, rep("global", times = length(globals)))
             name <<- c(name, globals)
           }
@@ -125,12 +136,20 @@ find_globals_ordered <- function(expr, envir, ..., trace = FALSE) {
   if (is.function(expr)) {
     if (typeof(expr) != "closure") return(character(0L)) ## e.g. `<-`
     fun <- expr
-
     w <- makeUsageCollector(fun, name = "<anonymous>",
                             enterLocal = enter_local,
                             enterGlobal = enter_global)
     if (trace) w <- inject_tracer_to_walker(w)
     collect_usage_function(fun, name = "<anonymous>", w)
+  } else if (is.call(expr) && is.function(expr[[1]])) {
+    ## AD HOC: Fixes https://github.com/HenrikBengtsson/globals/issues/60
+    for (e in list(expr[[1]], expr[-1])) {
+      globals <- find_globals_ordered(expr = e, envir = envir, ..., trace = trace)
+      if (length(globals) > 0) {
+        class <- c(class, rep("global", times = length(globals)))
+        name <- c(name, globals)
+      }
+    }
   } else {
     fun <- as_function(expr, envir = envir, ...)
     w <- makeUsageCollector(fun, name = "<anonymous>",
@@ -149,6 +168,58 @@ find_globals_ordered <- function(expr, envir, ..., trace = FALSE) {
 }
 
 
+call_find_globals_with_dotdotdot <- function(FUN, expr, envir, dotdotdot = "error", trace = FALSE, debug = FALSE) {
+  ## Is there a need for global '...', '..1', '..2', etc.?
+  dotdotdots <- character(0L)
+  
+  globals <- withCallingHandlers({
+    oopts <- options(warn = 0L)
+    on.exit(options(oopts))
+    FUN(expr, envir = envir, trace = trace)
+  }, warning = function(w) {
+    ## Warned about '...', '..1', '..2', etc.?
+    ## NOTE: The warning we're looking for is the one generated by
+    ## codetools::findGlobals().  That warning is _not_ translated,
+    ## meaning this approach should work as is as long as the message
+    ## is not modified by codetools itself.  If codetools ever changes
+    ## this such that the below string matching fails, then the package
+    ## tests (tests/dotdotdot.R) will detect that.  In other words,
+    ## such a change will not go unnoticed.  /HB 2017-03-08
+    msg <- w$message
+    pattern <- ".* ([.][.]([.]|[0-9]+)) may be used in an incorrect context.*"
+    if (grepl(pattern, msg, fixed = FALSE)) {
+      debug && mdebug(" - detected: %s", dQuote(trim(msg)))
+      if (dotdotdot %in% c("ignore", "return", "warning")) {
+        if (dotdotdot != "ignore") {
+          dotdotdots <<- c(dotdotdots, gsub(pattern, "\\1", msg))
+        }
+        if (dotdotdot != "warning") {
+          ## Consume / muffle warning
+          invokeRestart("muffleWarning")
+        }
+      } else if (dotdotdot == "error") {
+        e <- simpleError(msg, w$call)
+        stop(e)
+      }
+    }
+  })
+  
+  if (length(dotdotdots) > 0L) {
+    dotdotdots <- unique(dotdotdots)
+    globals <- c(globals, dotdotdots)
+  }
+  
+  globals
+}
+
+
+#' @param attributes If TRUE (default), attributes of `expr` are also searched.
+#' If FALSE, they are not.
+#' If a character vector, then attributes with matching names are searched.
+#' Note, the attributes of the attributes elements are not searched, that is,
+#' attributes are not searched recursively.  Also, attributes are searched
+#' with `dotdotdot = "ignore".
+#'
 #' @param dotdotdot TBD.
 #'
 #' @param trace TBD.
@@ -157,7 +228,9 @@ find_globals_ordered <- function(expr, envir, ..., trace = FALSE) {
 #'
 #' @rdname globalsOf
 #' @export
-findGlobals <- function(expr, envir = parent.frame(), ..., tweak = NULL,
+findGlobals <- function(expr, envir = parent.frame(), ...,
+                        attributes = TRUE,
+                        tweak = NULL,
                         dotdotdot = c("warning", "error", "return", "ignore"),
                         method = c("ordered", "conservative", "liberal"),
                         substitute = FALSE, unlist = TRUE, trace = FALSE) {
@@ -168,6 +241,13 @@ findGlobals <- function(expr, envir = parent.frame(), ..., tweak = NULL,
 
   debug <- mdebug("findGlobals(..., dotdotdot = '%s', method = '%s', unlist = %s) ...", dotdotdot, method, unlist)
 
+  if (is.logical(attributes)) {
+    stop_if_not(length(attributes) == 1L, !is.na(attributes))
+    if (!attributes) attributes <- character(0L)
+  } else {
+    stop_if_not(is.character(attributes), !anyNA(attributes))
+  }
+  
   if (is.list(expr)) {
     debug && mdebug(" - expr: <a list of length %d>", .length(expr))
 
@@ -193,8 +273,10 @@ findGlobals <- function(expr, envir = parent.frame(), ..., tweak = NULL,
       return(character(0L))
     }
     
-    globals <- list_apply(expr, FUN = findGlobals, envir = envir, ...,
+    globals <- list_apply(expr, FUN = findGlobals, envir = envir,
+                      attributes = attributes, ...,
                       tweak = tweak, dotdotdot = dotdotdot,
+                      method = method,
                       substitute = FALSE, unlist = FALSE)
     
     keep <- types <- NULL ## Not needed anymore
@@ -203,19 +285,11 @@ findGlobals <- function(expr, envir = parent.frame(), ..., tweak = NULL,
                     length(globals), hpaste(sQuote(names(globals))))
 
     if (unlist) {
-      needs_dotdotdot <- FALSE
-      for (kk in seq_along(globals)) {
-        s <- globals[[kk]]
-        n <- length(s)
-        if (identical(s[n], "...")) {
-          needs_dotdotdot <- TRUE
-          s <- s[-n]
-          globals[[kk]] <- s
-        }
-      }
       globals <- unlist(globals, use.names = FALSE)
       if (length(globals) > 1L) globals <- unique(globals)
-      if (needs_dotdotdot) globals <- c(globals, "...")
+      ## Move any ..., ..1, ..2, etc. to the very end
+      idxs <- grep("^[.][.]([.]|[0-9]+)$", globals)
+      if (length(idxs) > 0L) globals <- c(globals[-idxs], globals[idxs])
     }
 
     debug && mdebug(" - globals found: [%d] %s",
@@ -238,40 +312,34 @@ findGlobals <- function(expr, envir = parent.frame(), ..., tweak = NULL,
     find_globals_t <- find_globals_liberal
   }
 
-  ## Is there a need for global '...' variables?
-  needs_dotdotdot <- FALSE
-  globals <- withCallingHandlers({
-    oopts <- options(warn = 0L)
-    on.exit(options(oopts))
-    find_globals_t(expr, envir = envir, trace = trace)
-  }, warning = function(w) {
-    ## Warned about '...'?
-    ## NOTE: The warning we're looking for is the one generated by
-    ## codetools::findGlobals().  That warning is _not_ translated,
-    ## meaning this approach should work as is as long as the message
-    ## is not modified by codetools itself.  If codetools ever changes
-    ## this such that the below string matching fails, then the package
-    ## tests (tests/dotdotdot.R) will detect that.  In other words,
-    ## such a change will not go unnoticed.  /HB 2017-03-08
-    pattern <- "... may be used in an incorrect context"
-    if (grepl(pattern, w$message, fixed = TRUE)) {
-      debug && mdebug(" - detected: %s", dQuote(trim(w$message)))
-      needs_dotdotdot <<- TRUE
-      if (dotdotdot == "return") {
-        ## Consume / muffle warning
-        invokeRestart("muffleWarning")
-      } else if (dotdotdot == "ignore") {
-        needs_dotdotdot <<- FALSE
-        ## Consume / muffle warning
-        invokeRestart("muffleWarning")
-      } else if (dotdotdot == "error") {
-        e <- simpleError(w$message, w$call)
-        stop(e)
-      }
+  globals <- call_find_globals_with_dotdotdot(find_globals_t, expr = expr, envir = envir, dotdotdot = dotdotdot, trace = trace, debug = debug)
+  
+  ## Search attributes?
+  if (length(attributes) > 0) {
+    attrs <- attributes(expr)
+    if (is.character(attributes)) {
+      attrs <- attrs[names(attrs) %in% attributes]
     }
-  })
 
-  if (needs_dotdotdot) globals <- c(globals, "...")
+    ## Attributes to be searched, if any
+    if (length(attrs) > 0) {
+      debug && mdebug(" - searching attributes")
+      attrs_globals <- list_apply(attrs, FUN = findGlobals, envir = envir,
+                                  ## Don't look for attributes recursively
+                                  attributes = FALSE,
+                                  tweak = tweak,
+                                  ...,
+                                  ## Don't complain about '...', '..1', etc.
+                                  dotdotdot = "ignore",
+                                  method = method,
+                                  substitute = FALSE, unlist = FALSE)
+      if (unlist) attrs_globals <- unlist(attrs_globals, use.names = FALSE)
+      if (length(attrs_globals) > 1L) attrs_globals <- unique(attrs_globals)
+      debug && mdebug(" - globals found in attributes: [%d] %s",
+                      length(attrs_globals), hpaste(sQuote(attrs_globals)))
+      globals <- unique(c(globals, attrs_globals))
+    }
+  }
 
   debug && mdebug(" - globals found: [%d] %s", length(globals), hpaste(sQuote(globals)))
   debug && mdebug("findGlobals(..., dotdotdot = '%s', method = '%s', unlist = %s) ... DONE", dotdotdot, method, unlist) #nolint
@@ -301,7 +369,8 @@ collect_usage_function <- function(fun, name, w) {
   parnames <- names(formals)
 
   formals_clean <- drop_missing_formals(formals)
-  locals <- findLocalsList(c(list(body), formals_clean))
+#  locals <- findLocalsList(c(list(body), formals_clean))
+  locals <- findLocalsList(formals_clean)
 
   w$env <- new.env(hash = TRUE, parent = w$env)
   for (n in c(parnames, locals)) assign(n, TRUE, w$env)
@@ -315,14 +384,27 @@ inject_tracer_to_function <- function(fcn, name) {
   b <- body(fcn)
   f <- formals(fcn)
   args <- setdiff(names(f), c("w", "..."))
-  title <- sprintf("%s():", name)
+  if (length(args) > 0L) {
+    args <- grep("^[.][.][0-9]+$", args, invert = TRUE, value = TRUE)
+  }
+  title <- sprintf("%s()", name)
   b <- bquote({
-    message(.(title))
-    if (length(.(args)) > 0) {
-      mstr <- get("mstr", envir = getNamespace("globals"), mode = "function")
-      mstr(mget(.(args)))
+    message(.(title), ":")
+    if (length(.(args)) > 0)
+      message(paste(utils::capture.output(utils::str(mget(.(args)))), collapse = "\n"))
+    if (!exists("w", mode = "list")) return()
+    env <- environment(w$enterLocal)
+    n <- length(env$name)
+    value <- .(b)
+    nnew <- (length(env$name) - n)
+    if (nnew) {
+      message(" ", .(title), " variables:")
+      vars <- data.frame(name=env$name, class=env$class, stringsAsFactors = FALSE)
+      vars$added <- c(rep(FALSE, times = n), rep(TRUE, times = nnew))
+      message(paste(utils::capture.output(print(vars)), collapse = "\n"))
     }
-    .(b)
+    message(" ", .(title) , " => ", utils::capture.output(utils::str(value)))
+    value
   })
   body(fcn) <- b
   fcn
