@@ -47,6 +47,32 @@ is_internal <- function(x) {
   any(grepl(".Internal", body, fixed = TRUE))
 }
 
+# Example: base::.C_R_removeTaskCallback
+is_native_symbol_info <- function(x) {
+  if (!inherits(x, "NativeSymbolInfo")) return(FALSE)
+  if (typeof(x) != "list") return(FALSE)
+  address <- x$address
+  if (!inherits(address, "RegisteredNativeSymbol")) return(FALSE)
+  TRUE
+}
+
+isPackageNamespace <- function(env) {
+  if (!is.environment(env)) return(FALSE)
+  name <- environmentName(env)
+  if (name == "base") return(TRUE)
+  packageName <- env$.packageName
+  if (identical(name, packageName)) return(TRUE)
+  if (!grepl("^package:", name)) return(FALSE)
+  (name %in% search())
+}
+
+# From future 1.18.0
+asPkgEnvironment <- function(pkg) {
+  name <- sprintf("package:%s", pkg)
+  if (!name %in% search()) return(emptyenv())
+  as.environment(name)
+}
+
 ## From R.utils 2.0.2 (2015-05-23)
 hpaste <- function(..., sep="", collapse=", ", last_collapse=NULL,
                    max_head=if (missing(last_collapse)) 3 else Inf,
@@ -117,10 +143,24 @@ mstr <- function(...) {
 
 #' @importFrom utils capture.output
 envname <- function(env) {
+  if (!is.environment(env)) return(NA_character_)
   name <- environmentName(env)
   if (name == "") {
-    ## e.g. new.env()
-    name <- capture.output(print(env))
+    class <- class(env)
+    if (identical(class, "environment")) {
+      ## e.g. new.env()
+      name <- capture.output(print(env))
+    } else {
+      ## It might be that 'env' is on a class that extends 'environment',
+      ## e.g. R.oo::Object() or R6::R6Class().
+      ## IMPORTANT: The unset class must be temporary, because changing
+      ## the class of an environment will
+      name <- local({
+        on.exit(class(env) <- class)
+        class(env) <- NULL
+        capture.output(print(env))
+      })	
+    }
     name <- gsub("(.*: |>)", "", name)
   } else {
     ## e.g. globals:::where("plan")
@@ -204,4 +244,3 @@ list_apply <- function(X, FUN, ...) {
   }
   res
 }
-	
\ No newline at end of file
