@@ -1,11 +1,20 @@
-library("globals")
-opts <- options(warn = 1L)
+source("incl/start.R")
+
+options(warn = 1L)
 
 exprs <- list(
-  ok   = substitute(function(...) sum(x, ...)),
-  warn = substitute(sum(x, ...))
+  ok1   = quote(function(...) sum(x, ...)),
+  warn1 = quote(sum(x, ...)),
+  ok2   = quote(function(...) sum(x, ..1, ..2, ..3)),
+  warn2 = quote(sum(x, ..1, ..2, ..3))
 )
 
+truth <- list(
+  ok1 = c("sum", "x"),
+  ok2 = c("sum", "x"),
+  warn1 = c("sum", "x", "..."),
+  warn2 = c("sum", "x", "..1", "..2", "..3")
+)
 
 message("*** findGlobals() ...")
 
@@ -18,45 +27,37 @@ for (name in names(exprs)) {
   print(fun)
   globals <- codetools::findGlobals(fun)
   print(globals)
-  stopifnot(all.equal(globals, c("sum", "x")))
+  assert_identical_sets(globals, c("sum", "x"))
 
   message("\n*** findGlobals(dotdotdot = 'ignore'):")
   cat(sprintf("Expression '%s':\n", name))
   print(expr)
   globals <- findGlobals(expr, dotdotdot = "ignore")
   print(globals)
-  stopifnot(all.equal(globals, c("sum", "x")))
+  assert_identical_sets(globals, c("sum", "x"))
 
   message("\n*** findGlobals(dotdotdot = 'return'):")
   cat(sprintf("Expression '%s':\n", name))
   print(expr)
   globals <- findGlobals(expr, dotdotdot = "return")
   print(globals)
-  if (name == "ok") {
-    stopifnot(all.equal(globals, c("sum", "x")))
-  } else {
-    stopifnot(all.equal(globals, c("sum", "x", "...")))
-  }
+  assert_identical_sets(globals, truth[[name]])
 
-  message("\n*** findGlobals(dotdotdot = 'warn'):")
+  message("\n*** findGlobals(dotdotdot = 'warning'):")
   cat(sprintf("Expression '%s':\n", name))
   print(expr)
-  globals <- findGlobals(expr, dotdotdot = "warn")
+  globals <- findGlobals(expr, dotdotdot = "warning")
   print(globals)
-  if (name == "ok") {
-    stopifnot(all.equal(globals, c("sum", "x")))
-  } else {
-    stopifnot(all.equal(globals, c("sum", "x", "...")))
-  }
+  assert_identical_sets(globals, truth[[name]])
 
   message("\n*** findGlobals(dotdotdot = 'error'):")
   cat(sprintf("Expression '%s':\n", name))
   print(expr)
-  globals <- try(findGlobals(expr, dotdotdot = "error"))
-  if (name == "ok") {
-    stopifnot(all.equal(globals, c("sum", "x")))
+  globals <- tryCatch(findGlobals(expr, dotdotdot = "error"), error = identity)
+  if (name %in% c("ok1", "ok2")) {
+    assert_identical_sets(globals, truth[[name]])
   } else {
-    stopifnot(inherits(globals, "try-error"))
+    stopifnot(inherits(globals, "error"))
   }
 } # for (name ...)
 
@@ -64,7 +65,7 @@ message("\n*** findGlobals(<exprs>, dotdotdot = 'return'):")
 print(exprs)
 globals <- findGlobals(exprs, dotdotdot = "return")
 print(globals)
-
+assert_identical_sets(globals, unique(unlist(truth, use.names = FALSE)))
 
 message("*** findGlobals() ... DONE")
 
@@ -82,7 +83,7 @@ for (name in names(exprs)) {
   print(expr)
   globals <- globalsOf(expr, dotdotdot = "ignore")
   print(globals)
-  stopifnot(all.equal(names(globals), c("sum", "x")))
+  assert_identical_sets(names(globals), c("sum", "x"))
   stopifnot(all.equal(globals$sum, base::sum))
   stopifnot(all.equal(globals$x, x))
 
@@ -91,24 +92,20 @@ for (name in names(exprs)) {
   print(expr)
   globals <- globalsOf(expr, dotdotdot = "return")
   print(globals)
-  if (name == "ok") {
-    stopifnot(all.equal(names(globals), c("sum", "x")))
-  } else {
-    stopifnot(all.equal(names(globals), c("sum", "x", "...")))
+  assert_identical_sets(names(globals), truth[[name]])
+  if (name == "warn1") {
     stopifnot(!is.list(globals$`...`) && is.na(globals$`...`))
   }
   stopifnot(all.equal(globals$sum, base::sum))
   stopifnot(all.equal(globals$x, x))
 
-  message("\n*** globalsOf(dotdotdot = 'warn'):")
+  message("\n*** globalsOf(dotdotdot = 'warning'):")
   cat(sprintf("Expression '%s':\n", name))
   print(expr)
-  globals <- globalsOf(expr, dotdotdot = "warn")
+  globals <- globalsOf(expr, dotdotdot = "warning")
   print(globals)
-  if (name == "ok") {
-    stopifnot(all.equal(names(globals), c("sum", "x")))
-  } else {
-    stopifnot(all.equal(names(globals), c("sum", "x", "...")))
+  assert_identical_sets(names(globals), truth[[name]])
+  if (name == "warn1") {
     stopifnot(!is.list(globals$`...`) && is.na(globals$`...`))
   }
   stopifnot(all.equal(globals$sum, base::sum))
@@ -117,13 +114,13 @@ for (name in names(exprs)) {
   message("\n*** globalsOf(dotdotdot = 'error'):")
   cat(sprintf("Expression '%s':\n", name))
   print(expr)
-  globals <- try(globalsOf(expr, dotdotdot = "error"))
-  if (name == "ok") {
-    stopifnot(all.equal(names(globals), c("sum", "x")))
+  globals <- tryCatch(globalsOf(expr, dotdotdot = "error"), error = identity)
+  if (name %in% c("ok1", "ok2")) {
+    assert_identical_sets(names(globals), truth[[name]])
     stopifnot(all.equal(globals$sum, base::sum))
     stopifnot(all.equal(globals$x, x))
   } else {
-    stopifnot(inherits(globals, "try-error"))
+    stopifnot(inherits(globals, "error"))
   }
 } # for (name ...)
 
@@ -149,7 +146,7 @@ for (name in names(exprs)) {
   print(expr)
   globals <- globalsOf(expr, dotdotdot = "ignore")
   print(globals)
-  stopifnot(all.equal(names(globals), c("sum", "x")))
+  assert_identical_sets(names(globals), c("sum", "x"))
   stopifnot(all.equal(globals$sum, base::sum))
   stopifnot(all.equal(globals$x, x))
 
@@ -158,24 +155,20 @@ for (name in names(exprs)) {
   print(expr)
   globals <- globalsOf(expr, dotdotdot = "return")
   print(globals)
-  if (name == "ok") {
-    stopifnot(all.equal(names(globals), c("sum", "x")))
-  } else {
-    stopifnot(all.equal(names(globals), c("sum", "x", "...")))
+  assert_identical_sets(names(globals), truth[[name]])
+  if (name == "warn1") {
     stopifnot(all.equal(globals$`...`, args, check.attributes = FALSE))
   }
   stopifnot(all.equal(globals$sum, base::sum))
   stopifnot(all.equal(globals$x, x))
 
-  message("\n*** globalsOf(dotdotdot = 'warn'):")
+  message("\n*** globalsOf(dotdotdot = 'warning'):")
   cat(sprintf("Expression '%s':\n", name))
   print(expr)
-  globals <- globalsOf(expr, dotdotdot = "warn")
+  globals <- globalsOf(expr, dotdotdot = "warning")
   print(globals)
-  if (name == "ok") {
-    stopifnot(all.equal(names(globals), c("sum", "x")))
-  } else {
-    stopifnot(all.equal(names(globals), c("sum", "x", "...")))
+  assert_identical_sets(names(globals), truth[[name]])
+  if (name == "warn1") {
     stopifnot(all.equal(globals$`...`, args, check.attributes = FALSE))
   }
   stopifnot(all.equal(globals$sum, base::sum))
@@ -184,13 +177,13 @@ for (name in names(exprs)) {
   message("\n*** globalsOf(dotdotdot = 'error'):")
   cat(sprintf("Expression '%s':\n", name))
   print(expr)
-  globals <- try(globalsOf(expr, dotdotdot = "error"))
-  if (name == "ok") {
-    stopifnot(all.equal(names(globals), c("sum", "x")))
+  globals <- tryCatch(globalsOf(expr, dotdotdot = "error"), error = identity)
+  if (name %in% c("ok1", "ok2")) {
+    assert_identical_sets(names(globals), truth[[name]])
     stopifnot(all.equal(globals$sum, base::sum))
     stopifnot(all.equal(globals$x, x))
   } else {
-    stopifnot(inherits(globals, "try-error"))
+    stopifnot(inherits(globals, "error"))
   }
 } # for (name ...)
 
@@ -205,5 +198,5 @@ aux(x = 3:4, y = 1, z = 42L, exprs = exprs)
 message("*** function(x, ...) globalsOf() ... DONE")
 
 
-## Undo
-options(opts)
+## Cleanup
+source("incl/end.R")
